@@ -1,27 +1,24 @@
 package org.gcube.portlets.widgets.ckandatapublisherwidget.server.utils;
 
-import java.io.BufferedReader;
-import java.io.InputStream;
-import java.io.InputStreamReader;
+import static org.gcube.common.authorization.client.Constants.authorizationService;
+
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpSession;
 
-import org.apache.http.HttpEntity;
-import org.apache.http.HttpResponse;
-import org.apache.http.client.methods.HttpPost;
-import org.apache.http.entity.StringEntity;
-import org.apache.http.impl.client.CloseableHttpClient;
-import org.apache.http.impl.client.HttpClientBuilder;
-import org.gcube.application.framework.core.session.ASLSession;
+import org.gcube.common.authorization.client.exceptions.ObjectNotFound;
 import org.gcube.common.authorization.library.provider.SecurityTokenProvider;
+import org.gcube.common.authorization.library.provider.UserInfo;
 import org.gcube.common.homelibrary.home.exceptions.InternalErrorException;
 import org.gcube.common.homelibrary.home.workspace.WorkspaceItem;
 import org.gcube.common.homelibrary.home.workspace.folder.items.GCubeItem;
+import org.gcube.common.portal.PortalContext;
 import org.gcube.common.scope.api.ScopeProvider;
+import org.gcube.datacatalogue.ckanutillibrary.ApplicationProfileScopePerUrlReader;
 import org.gcube.datacatalogue.ckanutillibrary.DataCatalogue;
 import org.gcube.datacatalogue.ckanutillibrary.models.RolesCkanGroupOrOrg;
 import org.gcube.datacatalogue.ckanutillibrary.utils.SessionCatalogueAttributes;
@@ -48,11 +45,19 @@ import org.gcube.vomanagement.usermanagement.impl.LiferayRoleManager;
 import org.gcube.vomanagement.usermanagement.impl.LiferayUserManager;
 import org.gcube.vomanagement.usermanagement.model.GCubeGroup;
 import org.gcube.vomanagement.usermanagement.model.GCubeRole;
+import org.gcube.vomanagement.usermanagement.model.GCubeUser;
 import org.gcube.vomanagement.usermanagement.model.GatewayRolesNames;
 import org.slf4j.LoggerFactory;
 
 import com.fasterxml.jackson.databind.ObjectMapper;
 
+import eu.trentorise.opendata.jackan.internal.org.apache.http.HttpEntity;
+import eu.trentorise.opendata.jackan.internal.org.apache.http.HttpResponse;
+import eu.trentorise.opendata.jackan.internal.org.apache.http.client.methods.HttpPost;
+import eu.trentorise.opendata.jackan.internal.org.apache.http.entity.StringEntity;
+import eu.trentorise.opendata.jackan.internal.org.apache.http.impl.client.CloseableHttpClient;
+import eu.trentorise.opendata.jackan.internal.org.apache.http.impl.client.HttpClientBuilder;
+import eu.trentorise.opendata.jackan.internal.org.apache.http.util.EntityUtils;
 import eu.trentorise.opendata.jackan.model.CkanOrganization;
 
 /**
@@ -64,10 +69,11 @@ public class Utils {
 	// Logger
 	private static final org.slf4j.Logger logger = LoggerFactory.getLogger(Utils.class);
 	private static final String APPLICATION_ID_CATALOGUE_MANAGER = "org.gcube.datacatalogue.ProductCatalogue"; 
-	private static final String NOTIFICATION_MESSAGE = "Dear members,<br>The product PRODUCT_TITLE has been just published by USER_FULLNAME.<br>You can find it here: PRODUCT_URL <br>";
+	private static final String NOTIFICATION_MESSAGE = "Dear members,<br>The product 'PRODUCT_TITLE' has been just published by USER_FULLNAME.<br>You can find it here: PRODUCT_URL <br>";
 	private static final String SOCIAL_SERVICE_APPLICATION_TOKEN = "2/tokens/generate-application-token/";
 	private static final String SOCIAL_SERVICE_WRITE_APPLICATION_POST = "2/posts/write-post-app/";
 	private static final String MEDIATYPE_JSON = "application/json";
+	public static final String GCUBE_REQUEST_URL = "gcube-request-url";
 
 	/** Gets the gcube item properties.
 	 *
@@ -278,20 +284,22 @@ public class Utils {
 	 * Retrieve the list of metadata beans
 	 * @return
 	 */
-	public static List<MetaDataProfileBean> getMetadataProfilesList(String scope, HttpSession httpSession, ASLSession aslSession) {
+	public static List<MetaDataProfileBean> getMetadataProfilesList(String scope, HttpServletRequest request) {
 
 		List<MetaDataProfileBean> beans = new ArrayList<MetaDataProfileBean>();
-		String username = aslSession.getUsername();
+		String username = getCurrentUser(request).getUsername();
 		logger.debug("User in session is " + username);
 
 		// check the scope we need to discover
-		String scopeInWhichDiscover = (scope != null && !scope.isEmpty()) ? scope : aslSession.getScope();
+		String scopeInWhichDiscover = (scope != null && !scope.isEmpty()) ? scope : getCurrentContext(request, false);
 
 		logger.debug("Discovering into scope " + scopeInWhichDiscover);
 
 		// scope in which we need to discover
 		String keyPerScope = UtilMethods.concatenateSessionKeyScope(SessionCatalogueAttributes.CKAN_PROFILES_KEY, scopeInWhichDiscover);
 
+		HttpSession httpSession = request.getSession();
+
 		if(httpSession.getAttribute(keyPerScope) != null){
 			beans = (List<MetaDataProfileBean>)httpSession.getAttribute(keyPerScope);
 			logger.info("List of profiles was into session");
@@ -457,21 +465,11 @@ public class Utils {
 		if (entity != null) {
 			try {
 				toReturn = new HashMap<String, Object>();
-				InputStream is = entity.getContent();
-				BufferedReader reader = new BufferedReader(new InputStreamReader(is));
-				StringBuilder sb = new StringBuilder();
-
-				String line = null;
-
-				while ((line = reader.readLine()) != null) {
-					sb.append(line + "\n");
-				}
-
-				logger.debug("Response as string is " + sb.toString());
+				String jsonString = EntityUtils.toString(response.getEntity());
+				logger.debug("Response as string is " + jsonString);
 				ObjectMapper objectMapper = new ObjectMapper();
-				toReturn = objectMapper.readValue(sb.toString(), HashMap.class);
+				toReturn = objectMapper.readValue(jsonString, HashMap.class);
 				logger.debug("Map is " + toReturn);
-
 			}catch(Exception e){
 				logger.error("Failed to read json object", e);
 			}
@@ -479,4 +477,124 @@ public class Utils {
 
 		return toReturn;
 	}
+
+	/**
+	 * First check to retrieve the token, else create it
+	 * @param username
+	 * @param context
+	 * @return the user token for the context
+	 */
+	public static String tryGetElseCreateToken(String username, String context) {
+		String token = null;
+		try{
+			try{
+				logger.debug("Checking if token for user " + username + " in context " + context + " already exists...");
+				token = authorizationService().resolveTokenByUserAndContext(username, context);
+				logger.debug("It exists!");
+			}catch(ObjectNotFound e){
+				logger.info("Creating token for user " + username + " and context " + context);
+				token = authorizationService().generateUserToken(new UserInfo(username, new ArrayList<String>()), context);
+				logger.debug("received token: "+ token.substring(0, 5) + "***********************");
+			}
+		}catch(Exception e){
+			logger.error("Failed both token retrieval and creation", e);
+		}
+		return token;
+	}
+
+	/**
+	 * Get the scope in which ckan information needs to be discovered from the url
+	 * @param httpServletRequest
+	 * @return
+	 */
+	public static String getScopeFromClientUrl(HttpServletRequest httpServletRequest){
+
+		if(httpServletRequest == null)
+			throw new IllegalArgumentException("HttpServletRequest is null!");
+
+		String scopeToReturn = null;
+		try{
+			String clientUrl = getCurrentClientUrl(httpServletRequest).split("\\?")[0];
+			logger.debug("Client url is " + clientUrl);
+
+			// check if this information is in session, otherwise set it and return
+			HttpSession session = httpServletRequest.getSession();
+
+			if((scopeToReturn = (String) session.getAttribute(clientUrl)) != null){
+				logger.debug("Scope to return is " + scopeToReturn);
+			}else{
+				// ask to the ckan library and set it
+				scopeToReturn = ApplicationProfileScopePerUrlReader.getScopePerUrl(clientUrl);
+				logger.debug("Scope to return is " + scopeToReturn);
+				session.setAttribute(clientUrl, scopeToReturn);
+			}
+		}catch(Exception e){
+			scopeToReturn = getCurrentContext(httpServletRequest, false);
+			logger.warn("Failed to determine the scope from the client url, returning the current one: " + scopeToReturn);
+		}
+		return scopeToReturn;
+	}
+
+	/**
+	 * Needed to get the url of the client
+	 * @param httpServletRequest the httpServletRequest object
+	 * @return the instance of the user 
+	 * @see the url at client side
+	 */
+	public static String getCurrentClientUrl(HttpServletRequest httpServletRequest) {
+		return httpServletRequest.getHeader(GCUBE_REQUEST_URL);
+	}
+
+	/**
+	 * Retrieve the current user by using the portal manager
+	 * @return a GcubeUser object
+	 */
+	public static GCubeUser getCurrentUser(HttpServletRequest request){
+
+		if(request == null)
+			throw new IllegalArgumentException("HttpServletRequest is null!");
+
+		PortalContext pContext = PortalContext.getConfiguration();
+		GCubeUser user = pContext.getCurrentUser(request);
+		logger.debug("Returning user " + user);
+		return user;
+	}
+
+	/**
+	 * Retrieve the current scope by using the portal manager
+	 * @param b 
+	 * @return a GcubeUser object
+	 */
+	public static String getCurrentContext(HttpServletRequest request, boolean setInThread){
+
+		if(request == null)
+			throw new IllegalArgumentException("HttpServletRequest is null!");
+
+		PortalContext pContext = PortalContext.getConfiguration();
+		String context = pContext.getCurrentScope(request);
+		logger.debug("Returning context " + context);
+
+		if(context != null && setInThread)
+			ScopeProvider.instance.set(context);
+
+		return context;
+	}
+
+	/**
+	 * Retrieve the group given the scope
+	 * @param scope
+	 * @return
+	 * @throws UserManagementSystemException
+	 * @throws GroupRetrievalFault
+	 */
+	public static GCubeGroup getGroupFromScope(String scope) throws UserManagementSystemException, GroupRetrievalFault{
+
+		if(scope == null || scope.isEmpty())
+			throw new IllegalArgumentException("Scope is missing here!!");
+
+		GroupManager gm = new LiferayGroupManager();
+		long groupId = gm.getGroupIdFromInfrastructureScope(scope);
+		return gm.getGroup(groupId);
+
+	}
 }
\ No newline at end of file
